"""Analyze the Illumina DRAGEN results output files."""

import click
import logging
import os
import pathlib
import sys

from datetime import datetime
from rich.console import Console

from dragen_results_utils import constants
from dragen_results_utils.analyzer import Analyzer
from dragen_results_utils.file_utils import check_infile_status

DEFAULT_OUTDIR = os.path.join(
    constants.DEFAULT_OUTDIR_BASE,
    os.path.splitext(os.path.basename(__file__))[0],
    constants.DEFAULT_TIMESTAMP,
)

error_console = Console(stderr=True, style="bold red")

console = Console()


def validate_verbose(ctx, param, value):
    """Validate the validate option.

    Args:
        ctx (Context): The click context.
        param (str): The parameter.
        value (bool): The value.

    Returns:
        bool: The value.
    """

    if value is None:
        click.secho(
            "--verbose was not specified and therefore was set to 'True'", fg="yellow"
        )
        return constants.DEFAULT_VERBOSE
    return value


@click.command()
@click.option("--figure_file", help="The output figure file.")
@click.option("--logfile", help="The log file.")
@click.option(
    "--metrics_file",
    help="The input variant metrics file i.e.: .variant_metrics.csv or .variant_metrics.csv.gz.",
)
@click.option(
    "--metrics_summary_report_file",
    type=click.Path(exists=True),
    help="The output metrics summary report file.",
)
@click.option(
    "--outdir",
    help=f"The default is the current working directory - default is '{DEFAULT_OUTDIR}'.",
)
@click.option(
    "--variant_summary_report_file",
    type=click.Path(exists=True),
    help="The output variant summary report file.",
)
@click.option(
    "--vcf_file",
    type=click.Path(exists=True),
    help="The input variant file i.e.: .vcf or .vcf.gz.",
)
@click.option(
    "--verbose",
    is_flag=True,
    help=f"Will print more info to STDOUT - default is '{constants.DEFAULT_VERBOSE}'.",
    callback=validate_verbose,
)
def main(
    figure_file: str,
    logfile: str,
    metrics_file: str,
    metrics_summary_report_file: str,
    outdir: str,
    variant_summary_report_file: str,
    vcf_file: str,
    verbose: bool,
):
    """Analyze the Illumina DRAGEN results output files."""

    error_ctr = 0

    if vcf_file is None:
        error_console.print("--vcf_file was not specified")
        error_ctr += 1

    if metrics_file is None:
        error_console.print("--metrics_file was not specified")
        error_ctr += 1

    if error_ctr > 0:
        click.echo(click.get_current_context().get_help())
        sys.exit(1)

    check_infile_status(vcf_file)
    check_infile_status(metrics_file)

    if outdir is None:
        outdir = DEFAULT_OUTDIR
        console.print(
            f"[yellow]--outdir was not specified and therefore was set to '{outdir}'[/]"
        )

    if not os.path.exists(outdir):
        pathlib.Path(outdir).mkdir(parents=True, exist_ok=True)
        console.print(f"[yellow]Created output directory '{outdir}'[/]")

    if logfile is None:
        logfile = os.path.join(
            outdir, os.path.splitext(os.path.basename(__file__))[0] + ".log"
        )
        console.print(
            f"[yellow]--logfile was not specified and therefore was set to '{logfile}'[/]"
        )

    if figure_file is None:
        figure_file = os.path.join(
            outdir,
            os.path.splitext(os.path.basename(vcf_file))[0]
            + ".quality_distribution.png",
        )
        console.print(
            f"[yellow]--figure_file was not specified and therefore was set to '{figure_file}'[/]"
        )

    if variant_summary_report_file is None:
        variant_summary_report_file = os.path.join(
            outdir,
            os.path.splitext(os.path.basename(vcf_file))[0]
            + ".variant_summary_report.txt",
        )
        console.print(
            f"[yellow]--variant_summary_report_file was not specified and therefore was set to '{variant_summary_report_file}'[/]"
        )

    if metrics_summary_report_file is None:
        metrics_summary_report_file = os.path.join(
            outdir,
            os.path.splitext(os.path.basename(vcf_file))[0]
            + ".metrics_summary_report.txt",
        )
        console.print(
            f"[yellow]--metrics_summary_report_file was not specified and therefore was set to '{metrics_summary_report_file}'[/]"
        )

    logging.basicConfig(
        filename=logfile,
        format=constants.DEFAULT_LOGGING_FORMAT,
        level=constants.DEFAULT_LOGGING_LEVEL,
    )

    analyzer = Analyzer(
        figure_file=figure_file,
        metrics_file=metrics_file,
        metrics_summary_report_file=metrics_summary_report_file,
        logfile=logfile,
        outdir=outdir,
        variant_summary_report_file=variant_summary_report_file,
        vcf_file=vcf_file,
        verbose=verbose,
    )

    analyzer.analyze_results()

    if verbose:
        print(f"The log file is '{logfile}'")
        console.print(
            f"[bold green]Execution of '{os.path.abspath(__file__)}' completed[/]"
        )


if __name__ == "__main__":
    main()
