"""
DRAGENOutputProcessor: A Python tool to process and analyze Illumina DRAGEN output files
Author: [Your Name]
Date: February 26, 2025
Description: Processes DRAGEN VCF and variant metrics files (compressed or uncompressed) to generate summary statistics and visualizations
"""

import logging
import gzip
import os

import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns

from datetime import datetime

from typing import Any, Dict

from dragen_results_utils import constants


class Analyzer:
    def __init__(self, **kwargs):
        """Initialize the processor with input file paths and output directory."""
        self.figure_file = kwargs.get("figure_file", None)
        self.metrics_file = kwargs.get("metrics_file", None)
        self.metrics_summary_report_file = kwargs.get(
            "metrics_summary_report_file", None
        )
        self.logfile = kwargs.get("logfile", None)
        self.outdir = kwargs.get("outdir", None)
        self.variant_summary_report_file = kwargs.get(
            "variant_summary_report_file", None
        )
        self.vcf_file = kwargs.get("vcf_file", None)
        self.verbose = kwargs.get("verbose", constants.DEFAULT_VERBOSE)

        if self.figure_file is None:
            self.figure_file = os.path.join(self.outdir, "quality_distribution.png")
            logging.info(
                f"Figure file not specified and therefore was set to '{self.figure_file}'"
            )

        if self.metrics_summary_report_file is None:
            self.metrics_summary_report_file = os.path.join(
                self.outdir, "metrics_summary.txt"
            )
            logging.info(
                f"Output file not specified and therefore was set to '{self.metrics_summary_report_file}'"
            )

        if not os.path.exists(self.outdir):
            self.outdir.mkdir(parents=True, exist_ok=True)

        # Validate file extensions
        if self.vcf_file.endswith(".vcf") or self.vcf_file.endswith(".vcf.gz"):
            logging.info(f"Will process VCF file: {self.vcf_file}")
        else:
            raise ValueError("VCF file must be .vcf or .vcf.gz")

        if self.metrics_file.endswith(
            ".variant_metrics.csv"
        ) or self.self.metrics_file.endswith(".variant_metrics.csv.gz"):
            logging.info(f"Will process metrics file: {self.metrics_file}")
        else:
            raise ValueError(
                "Metrics file must be .variant_metrics.csv or .variant_metrics.csv.gz"
            )

        logging.info(f"Instantiated Analyzer class in '{os.path.abspath(__file__)}'")

    def analyze_results(self) -> None:
        """Main processing pipeline.

        Raises:
            Exception: If an error occurs during processing.
        """
        try:
            logging.info("Starting DRAGEN results output processing")

            # Process VCF file
            variants_df = self._parse_vcf()

            # Process metrics file
            metrics = self._parse_metrics()

            # Generate analysis outputs
            self._generate_variant_summary(variants_df)

            self._write_metrics_summary_report(metrics)

        except Exception as e:
            logging.error(f"Processing failed: {str(e)}")
            raise

    def _write_metrics_summary_report(self, metrics: Dict[str, Any]) -> None:
        """Write report file for summary metrics.

        Args:
            metrics (dict): Dictionary of metrics to write to file
        """
        with open(self.metrics_summary_report_file, "w") as of:
            of.write(f"## method-created: {os.path.abspath(__file__)}\n")
            of.write(
                f"## date-created: {str(datetime.today().strftime('%Y-%m-%d-%H%M%S'))}\n"
            )
            of.write(f"## created-by: {os.environ.get('USER')}\n")
            of.write(f"## vcf-file: {self.vcf_file}\n")
            of.write(f"## metrics-file: {self.metrics_file}\n")
            of.write(f"## logfile: {self.logfile}\n")

            for key, value in metrics.items():
                of.write(f"{key}: {value}\n")

        logging.info(
            f"Wrote summary metrics report file '{self.metrics_summary_report_file}'"
        )
        if self.verbose:
            print(
                f"Wrote summary metrics report file '{self.metrics_summary_report_file}'"
            )

    def _parse_vcf(self) -> pd.DataFrame:
        """Parse DRAGEN VCF file (compressed or uncompressed) and extract key variant information.

        Returns:
            pd.DataFrame: DataFrame containing variant information.

        Raises:
            Exception: If an error occurs during parsing.
        """
        try:
            logging.info(f"Parsing VCF file: {self.vcf_file}")
            variants = []

            # Open file based on whether it's compressed
            opener = gzip.open if self.vcf_file.endswith(".gz") else open
            mode = "rt" if self.vcf_file.endswith(".gz") else "r"

            with opener(self.vcf_file, mode) as f:
                for line in f:
                    if line.startswith("#"):
                        continue
                    fields = line.strip().split("\t")
                    variant = {
                        "CHROM": fields[constants.DEFAULT_VCF_CHROM_INDEX],
                        "POS": int(fields[constants.DEFAULT_VCF_POS_INDEX]),
                        "REF": fields[constants.DEFAULT_VCF_REF_INDEX],
                        "ALT": fields[constants.DEFAULT_VCF_ALT_INDEX],
                        "QUAL": float(fields[constants.DEFAULT_VCF_QUAL_INDEX]),
                        "FILTER": fields[constants.DEFAULT_VCF_FILTER_INDEX],
                    }
                    variants.append(variant)

            df = pd.DataFrame(variants)
            logging.info(f"Found {len(df)} variants in VCF file")
            return df

        except Exception as e:
            logging.error(f"Error parsing VCF: {str(e)}")
            raise

    def _parse_metrics(self) -> Dict[str, float]:
        """Parse DRAGEN variant metrics file (compressed or uncompressed) and extract key statistics.

        Returns:
            Dict[str, float]: Dictionary of metrics extracted from the file.

        Raises:
            Exception: If an error occurs during parsing.
        """
        try:
            logging.info(f"Parsing metrics file: {self.metrics_file}")
            metrics = {}

            # Open file based on whether it's compressed
            opener = gzip.open if self.metrics_file.endswith(".gz") else open
            mode = "rt" if self.metrics_file.endswith(".gz") else "r"

            with opener(self.metrics_file, mode) as f:
                for line in f:
                    if line.strip() and not line.startswith("#"):
                        fields = line.strip().split(",")
                        if len(fields) >= 2:
                            key = fields[0].strip()
                            value = fields[1].strip()
                            try:
                                metrics[key] = float(value)
                            except ValueError:
                                metrics[key] = value
            return metrics

        except Exception as e:
            logging.error(f"Error parsing metrics: {str(e)}")
            raise

    def _generate_variant_summary(self, variants_df: pd.DataFrame) -> None:
        """Generate summary statistics and visualizations for variants.

        Args:
            variants_df (pd.DataFrame): DataFrame containing variant information.

        Raises:
            Exception: If an error occurs during generation.
        """
        try:
            # Basic statistics
            summary = {
                "Total_Variants": len(variants_df),
                "Passed_Filters": len(variants_df[variants_df["FILTER"] == "PASS"]),
                "Average_QUAL": variants_df["QUAL"].mean(),
            }

            # Save summary to file
            with open(self.variant_summary_report_file, "w") as f:
                f.write(f"## method-created: {os.path.abspath(__file__)}\n")
                f.write(
                    f"## date-created: {str(datetime.today().strftime('%Y-%m-%d-%H%M%S'))}\n"
                )
                f.write(f"## created-by: {os.environ.get('USER')}\n")
                f.write(f"## vcf-file: {self.vcf_file}\n")
                f.write(f"## metrics-file: {self.metrics_file}\n")
                f.write(f"## logfile: {self.logfile}\n")

                for key, value in summary.items():
                    f.write(f"{key}: {value}\n")

            # Create variant quality distribution plot
            plt.figure(figsize=(10, 6))
            sns.histplot(data=variants_df, x="QUAL", bins=50)
            plt.title("Distribution of Variant Quality Scores")
            plt.xlabel("Quality Score")
            plt.ylabel("Count")
            plt.savefig(self.figure_file)
            plt.close()

            logging.info(
                f"Generated variant summary and visualization '{self.figure_file}'"
            )

        except Exception as e:
            logging.error(f"Error generating variant summary: {str(e)}")
            raise

